// Copyright (C) 2013 S&P Capital IQ
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.spcapitaliq.platformexecuting

import java.io.File.{separator, pathSeparator}
import java.util.Locale.{ROOT => pivotLocale}

import scalaz.Id.Id
import scalaz.std.option._
import scalaz.std.string._
import scalaz.syntax.apply._
import scalaz.syntax.std.boolean._
import scalaz.syntax.equal._

/** A binary platform description.  I purposefully wash over most of
  * the actual complexity of platform-specific binaries, because it
  * doesn't interest me in this release.
  *
  * @param os An OS identifier.
  * @param arch The basic processor architecture.
  */
final case class Platform private(os: String, arch: String)

/** Platform detection and binary-compatible architecture picking. */
object Platform extends scala.runtime.AbstractFunction2[String, String, Platform] {
  /** Make a `Platform`. */
  def mk(os: String, arch: String) =
    Platform(classpathSafe(os), classpathSafe(arch))

  private val archAliases =
    Seq(("amd64", Seq("x86_64")),
        ("x86", Seq("i686", "i386")))

  private def unaliasArch(a: String): String =
    archAliases find (_._2 find (a ===) isDefined) map (_._1) getOrElse a

  /** Munge characters to make this nice for appearance in the
    * classpath. */
  private def classpathSafe(p: String): String =
    (p replaceAll ("""[ \\/:]""", "_")
       toUpperCase pivotLocale toLowerCase pivotLocale)

  /** Various platforms "compatible" with spec to try to retrieve a
    * captured binary for.
    *
    * @note Invariant: s ∉ compatPlatforms(s)
    */
  def compatPlatforms(spec: Platform): Seq[Platform] =
    spec match {
      case Platform(on, "amd64") => Seq(Platform(on, "x86"))
      case _ => Seq()
    }

  @inline private[this] def windows = "Windows" // magic number

  /** Name and CPU architecture of the current OS, if I can figure it
    * out.
    */
  def platformSpec: Option[Platform] =
    ^(Option(System getProperty "os.name")
        map {case v if v startsWith "Windows" => windows
             case v => v}
        orElse (separator === "\\" && pathSeparator === ";").option(windows),
      Option(System getProperty "os.arch") map unaliasArch)(mk)

  override def toString = "Platform"
}
